"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("../../../compiled/chalk"));
const chokidar_1 = __importDefault(require("../../../compiled/chokidar"));
const debug_1 = __importDefault(require("../../../compiled/debug"));
const fast_glob_1 = __importDefault(require("../../../compiled/fast-glob"));
const lodash_1 = __importDefault(require("../../../compiled/lodash"));
const rimraf_1 = __importDefault(require("../../../compiled/rimraf"));
const constants_1 = require("../../constants");
const logger = __importStar(require("../../logger"));
const winPath_1 = require("../../utils/winPath");
const dts_1 = __importDefault(require("./dts"));
const loaders_1 = __importDefault(require("./loaders"));
const debugLog = (0, debug_1.default)(constants_1.DEBUG_BUNDLESS_NAME);
const DEFAULT_BUNDLESS_IGNORES = [
    '**/*.md',
    '**/*.d.ts',
    '**/fixtures/**',
    '**/__{test,tests,snapshots}__/**',
    '**/*.{test,e2e,spec}.{js,jsx,ts,tsx}',
];
/**
 * replace extension for path
 */
function replacePathExt(filePath, ext) {
    const parsed = path_1.default.parse(filePath);
    return path_1.default.join(parsed.dir, `${parsed.name}${ext}`);
}
/**
 * transform specific files
 */
async function transformFiles(files, opts) {
    try {
        let count = 0;
        const declarationFileMap = new Map();
        // process all matched items
        for (let item of files) {
            const config = opts.configProvider.getConfigForFile(item);
            const itemAbsPath = path_1.default.join(opts.cwd, item);
            if (config) {
                let itemDistPath = path_1.default.join(config.output, path_1.default.relative(config.input, item));
                let itemDistAbsPath = path_1.default.join(opts.cwd, itemDistPath);
                const parentPath = path_1.default.dirname(itemDistAbsPath);
                // create parent directory if not exists
                if (!fs_1.default.existsSync(parentPath)) {
                    fs_1.default.mkdirSync(parentPath, { recursive: true });
                }
                // get result from loaders
                const result = await (0, loaders_1.default)(itemAbsPath, {
                    config,
                    pkg: opts.configProvider.pkg,
                    cwd: opts.cwd,
                });
                if (result) {
                    // update ext if loader specified
                    if (result.options.ext) {
                        itemDistPath = replacePathExt(itemDistPath, result.options.ext);
                        itemDistAbsPath = replacePathExt(itemDistAbsPath, result.options.ext);
                    }
                    // prepare for declaration
                    if (result.options.declaration) {
                        // use winPath because ts compiler will convert to posix path
                        declarationFileMap.set((0, winPath_1.winPath)(itemAbsPath), parentPath);
                    }
                    // distribute file with result
                    fs_1.default.writeFileSync(itemDistAbsPath, result.content);
                }
                else {
                    // copy file as normal assets
                    fs_1.default.copyFileSync(itemAbsPath, itemDistAbsPath);
                }
                if (!opts.quiet) {
                    logger.event(`Bundless ${chalk_1.default.gray(item)} to ${chalk_1.default.gray(itemDistPath)}${(result === null || result === void 0 ? void 0 : result.options.declaration) ? ' (with declaration)' : ''}`);
                }
                count += 1;
            }
            else {
                debugLog(`No config matches ${chalk_1.default.gray(item)}, skip`);
            }
        }
        if (declarationFileMap.size) {
            logger.event(`Generate declaration file${declarationFileMap.size > 1 ? 's' : ''}...`);
            const declarations = await (0, dts_1.default)([...declarationFileMap.keys()], {
                cwd: opts.cwd,
            });
            declarations.forEach((item) => {
                fs_1.default.writeFileSync(path_1.default.join(declarationFileMap.get(item.sourceFile), item.file), item.content, 'utf-8');
            });
        }
        return count;
    }
    catch (err) {
        if (opts.watch) {
            logger.error(err.message);
            return 0;
        }
        else {
            throw err;
        }
    }
}
async function bundless(opts) {
    logger.info(`Bundless for ${chalk_1.default.yellow(opts.configProvider.input)} directory to ${chalk_1.default.yellow(opts.configProvider.configs[0].format)} format`);
    const startTime = Date.now();
    const matches = fast_glob_1.default.sync(`${opts.configProvider.input}/**`, {
        cwd: opts.cwd,
        ignore: DEFAULT_BUNDLESS_IGNORES,
        nodir: true,
    });
    const count = await transformFiles(matches, opts);
    if (!opts.watch) {
        // output result for normal mode
        logger.event(`Transformed successfully in ${Date.now() - startTime} ms (${count} files)`);
    }
    else {
        // watching for watch mode
        logger.event(`Start watching ${opts.configProvider.input} directory...`);
        const watcher = chokidar_1.default
            .watch(opts.configProvider.input, {
            cwd: opts.cwd,
            ignoreInitial: true,
            ignored: DEFAULT_BUNDLESS_IGNORES,
        })
            .on('add', (rltFilePath) => {
            transformFiles([rltFilePath], opts);
        })
            .on('change', lodash_1.default.debounce((filePath) => {
            transformFiles([filePath], opts);
        }, constants_1.WATCH_DEBOUNCE_STEP, { leading: true, trailing: false }))
            .on('unlink', (rltFilePath) => {
            const isTsFile = /\.tsx?$/.test(rltFilePath);
            const config = opts.configProvider.getConfigForFile(rltFilePath);
            const fileDistAbsPath = path_1.default.join(opts.cwd, config.output, path_1.default.relative(config.input, rltFilePath));
            // TODO: collect real emit files
            const relatedFiles = isTsFile
                ? [
                    replacePathExt(fileDistAbsPath, '.js'),
                    replacePathExt(fileDistAbsPath, '.d.ts'),
                    replacePathExt(fileDistAbsPath, '.d.ts.map'),
                ]
                : [fileDistAbsPath];
            const relatedMainFile = relatedFiles.find((item) => fs_1.default.existsSync(item));
            if (relatedMainFile) {
                relatedFiles.forEach((file) => rimraf_1.default.sync(file));
                logger.event(`Bundless ${chalk_1.default.gray(path_1.default.relative(opts.cwd, relatedMainFile))} is removed`);
            }
        })
            .on('unlinkDir', (rltDirPath) => {
            const config = opts.configProvider.getConfigForFile(rltDirPath);
            const dirDistAbsPath = path_1.default.join(opts.cwd, config.output, path_1.default.relative(config.input, rltDirPath));
            if (fs_1.default.existsSync(dirDistAbsPath)) {
                rimraf_1.default.sync(dirDistAbsPath);
                // there are file removal logs above, so we don't need to log here
            }
        });
        return watcher;
    }
}
exports.default = bundless;
