import { Minimatch } from 'minimatch';
import { BuildTypes, BundlessTypes, MsupBaseConfig, MsupBundleConfig, MsupBundlessConfig, MsupConfig, PkgType } from '../types';
/**
 * declare bundler config
 */
export interface IBundleConfig extends MsupConfig, Omit<MsupBundleConfig, 'entry' | 'output'> {
    type: BuildTypes.BUNDLE;
    bundler: 'webpack';
    entry: string;
    output: {
        filename: string;
        path: string;
    };
}
/**
 * declare bundless config
 */
export interface IBundlessConfig extends MsupBaseConfig, Omit<MsupBundlessConfig, 'input' | 'overrides'> {
    type: BuildTypes.BUNDLESS;
    format: BundlessTypes;
    input: string;
}
/**
 * declare union builder config
 */
export declare type IBuilderConfig = IBundleConfig | IBundlessConfig;
/**
 *
 * convert alias from tsconfig paths
 * @export
 * @param {string} cwd
 */
export declare function convertAliasByTsconfigPaths(cwd: string): {
    bundle: Record<string, string>;
    bundless: Record<string, string>;
};
/**
 * normalize user config to bundler configs
 * @param userConfig  config from user
 * @param pkg
 */
export declare function normalizeUserConfig(userConfig: MsupConfig, pkg: PkgType): IBuilderConfig[];
declare class Minimatcher {
    matcher?: InstanceType<typeof Minimatch>;
    ignoreMatchers: InstanceType<typeof Minimatch>[];
    constructor(pattern: string, ignores?: string[]);
    match(filePath: string): boolean;
}
declare class ConfigProvider {
    pkg: ConstructorParameters<typeof ConfigProvider>[0];
    constructor(pkg: PkgType);
    onConfigChange(): void;
}
export declare class BundleConfigProvider extends ConfigProvider {
    type: BuildTypes;
    configs: IBundleConfig[];
    constructor(configs: IBundleConfig[], pkg: ConstructorParameters<typeof ConfigProvider>[0]);
}
export declare class BundlessConfigProvider extends ConfigProvider {
    type: BuildTypes;
    configs: IBundlessConfig[];
    input: string;
    output: string;
    matchers: InstanceType<typeof Minimatcher>[];
    constructor(configs: IBundlessConfig[], pkg: ConstructorParameters<typeof ConfigProvider>[0]);
    getConfigForFile(filePath: string): IBundlessConfig;
}
export declare function createConfigProviders(userConfig: MsupConfig, pkg: PkgType, cwd: string): {
    bundless: {
        esm?: BundlessConfigProvider;
        cjs?: BundlessConfigProvider;
    };
    bundle?: BundleConfigProvider | undefined;
};
export {};
