"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const rimraf_1 = __importDefault(require("../../compiled/rimraf"));
const logger = __importStar(require("../logger"));
const bundle_1 = __importDefault(require("./bundle"));
const bundless_1 = __importDefault(require("./bundless"));
const config_1 = require("./config");
function getProviderOutputs(providers) {
    var _a, _b, _c;
    const set = new Set();
    [
        (_a = providers.bundle) === null || _a === void 0 ? void 0 : _a.configs,
        (_b = providers.bundless.esm) === null || _b === void 0 ? void 0 : _b.configs,
        (_c = providers.bundless.cjs) === null || _c === void 0 ? void 0 : _c.configs,
    ].forEach((configs) => {
        configs === null || configs === void 0 ? void 0 : configs.forEach((config) => {
            set.add(typeof config.output === 'string' ? config.output : config.output.path);
        });
    });
    return Array.from(set);
}
async function builder(opts) {
    const configProviders = (0, config_1.createConfigProviders)(opts.userConfig, opts.pkg, opts.cwd);
    const outputs = getProviderOutputs(configProviders);
    const watchers = [];
    if (opts.clean !== false) {
        // clean output directories
        logger.info('Clean output directories');
        outputs.forEach((output) => {
            rimraf_1.default.sync(path_1.default.join(opts.cwd, output));
        });
    }
    if (!opts.watch && configProviders.bundle) {
        await (0, bundle_1.default)({
            cwd: opts.cwd,
            configProvider: configProviders.bundle,
        });
    }
    if (configProviders.bundless.esm) {
        const watcher = await (0, bundless_1.default)({
            cwd: opts.cwd,
            configProvider: configProviders.bundless.esm,
            watch: opts.watch,
            quiet: opts.quiet,
        });
        opts.watch && watchers.push(watcher);
    }
    if (configProviders.bundless.cjs) {
        const watcher = await (0, bundless_1.default)({
            cwd: opts.cwd,
            configProvider: configProviders.bundless.cjs,
            watch: opts.watch,
            quiet: opts.quiet,
        });
        opts.watch && watchers.push(watcher);
    }
    if (opts.watch) {
        return {
            async close() {
                await Promise.all(watchers.map((watcher) => watcher.close()));
            },
        };
    }
}
exports.default = builder;
