"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = exports.loadConfig = exports.defineConfig = void 0;
const path_1 = __importDefault(require("path"));
const esbuild_1 = __importDefault(require("../../compiled/esbuild"));
const fs_extra_1 = __importDefault(require("../../compiled/fs-extra"));
const lodash_1 = __importDefault(require("../../compiled/lodash"));
const constants_1 = require("../constants");
const register = __importStar(require("../register"));
const utils_1 = require("./utils");
function defineConfig(config) {
    return config;
}
exports.defineConfig = defineConfig;
function getUserConfig(configFiles) {
    let config = {};
    const files = [];
    for (const configFile of configFiles) {
        if (fs_extra_1.default.existsSync(configFile)) {
            register.register({
                implementor: esbuild_1.default,
            });
            register.clearFiles();
            config = lodash_1.default.merge(config, require(configFile).default);
            for (const file of register.getFiles()) {
                delete require.cache[file];
            }
            // includes the config File
            files.push(...register.getFiles());
            register.restore();
        }
        else {
            files.push(configFile);
        }
    }
    return {
        config: config,
        files,
    };
}
async function loadConfig() {
    const { config } = getUserConfig((0, utils_1.getAbsFiles)({
        files: constants_1.DEFAULT_CONFIG_FILES,
        cwd: process.cwd(),
    }));
    return config;
}
exports.loadConfig = loadConfig;
class Config {
    constructor(opts) {
        this.files = [];
        this.opts = opts;
        this.mainConfigFile = Config.getMainConfigFile(this.opts);
        this.prevConfig = null;
    }
    getConfig(opts = {}) {
        const { config, files } = this.getUserConfig();
        // Config.validateConfig({ config, schemas: opts.schemas });
        this.files = files;
        return (this.prevConfig = {
            config: config,
            files,
        });
    }
    // watch(opts: {
    //   schemas: ISchema;
    //   onChangeTypes: IOnChangeTypes;
    //   onChange: (opts: {
    //     data: ReturnType<typeof Config.diffConfigs>;
    //     event: string;
    //     path: string;
    //   }) => Promise<void>;
    // }) {
    //   const watcher = chokidar.watch(
    //     [
    //       ...this.files,
    //       ...(this.mainConfigFile
    //         ? []
    //         : getAbsFiles({
    //           files: this.opts.defaultConfigFiles || configFiles,
    //           cwd: this.opts.cwd,
    //         })),
    //     ],
    //     {
    //       ignoreInitial: true,
    //       cwd: this.opts.cwd,
    //     },
    //   );
    //   watcher.on(
    //     'all',
    //     lodash.debounce((event, path) => {
    //       const { config: origin } = this.prevConfig;
    //       const { config: updated, files } = this.getConfig({
    //         schemas: opts.schemas,
    //       });
    //       watcher.add(files);
    //       const data = Config.diffConfigs({
    //         origin,
    //         updated,
    //         onChangeTypes: opts.onChangeTypes,
    //       });
    //       opts
    //         .onChange({
    //           data,
    //           event,
    //           path,
    //         })
    //         .catch((e) => {
    //           throw new Error(e);
    //         });
    //     }, WATCH_DEBOUNCE_STEP),
    //   );
    //   return () => watcher.close();
    // }
    static getMainConfigFile(opts) {
        let mainConfigFile = null;
        for (const configFile of opts.defaultConfigFiles || constants_1.DEFAULT_CONFIG_FILES) {
            const absConfigFile = path_1.default.join(opts.cwd, configFile);
            if (fs_extra_1.default.existsSync(absConfigFile)) {
                mainConfigFile = absConfigFile;
                break;
            }
        }
        return mainConfigFile;
    }
    static getConfigFiles(opts) {
        const ret = [];
        const { mainConfigFile } = opts;
        const specifiedEnv = opts.specifiedEnv || '';
        if (mainConfigFile) {
            ret.push(...[
                mainConfigFile,
                specifiedEnv &&
                    (0, utils_1.addExt)({ file: mainConfigFile, ext: `.${specifiedEnv}` }),
                (0, utils_1.addExt)({ file: mainConfigFile, ext: '.local' }),
            ].filter(Boolean));
        }
        return ret;
    }
    getUserConfig() {
        const configFiles = Config.getConfigFiles({
            mainConfigFile: this.mainConfigFile,
            // env: this.opts.env,
            specifiedEnv: this.opts.specifiedEnv,
        });
        return Config.getUserConfig({
            configFiles: (0, utils_1.getAbsFiles)({
                files: configFiles,
                cwd: this.opts.cwd,
            }),
        });
    }
    static getUserConfig(opts) {
        let config = {};
        const files = [];
        for (const configFile of opts.configFiles) {
            if (fs_extra_1.default.existsSync(configFile)) {
                register.register({
                    implementor: esbuild_1.default,
                });
                register.clearFiles();
                config = lodash_1.default.merge(config, require(configFile).default);
                for (const file of register.getFiles()) {
                    delete require.cache[file];
                }
                // includes the config File
                files.push(...register.getFiles());
                register.restore();
            }
            else {
                files.push(configFile);
            }
        }
        return {
            config,
            files,
        };
    }
    static validateConfig(opts) {
        //     const errors = new Map<string, Error>();
        //     const configKeys = new Set(Object.keys(opts.config));
        //     for (const key of Object.keys(opts.schemas)) {
        //       configKeys.delete(key);
        //       if (!opts.config[key]) continue;
        //       const schema = opts.schemas[key](joi);
        //       // invalid schema
        //       assert(joi.isSchema(schema), `schema for config ${key} is not valid.`);
        //       const { error } = schema.validate(opts.config[key]);
        //       if (error) errors.set(key, error);
        //     }
        //     // invalid config values
        //     assert(
        //       errors.size === 0,
        //       `Invalid config values: ${Array.from(errors.keys()).join(', ')}
        // ${Array.from(errors.keys()).map((key) => {
        //         return `Invalid value for ${key}:\n${errors.get(key)!.message}`;
        //       })}`,
        //     );
        //     // invalid config keys
        //     assert(
        //       configKeys.size === 0,
        //       `Invalid config keys: ${Array.from(configKeys).join(', ')}`,
        //     );
    }
}
exports.Config = Config;
