"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseExtendsConfig = exports.getAbsFiles = exports.addExt = void 0;
const path_1 = require("path");
const deepmerge_1 = __importDefault(require("../../compiled/deepmerge"));
const resolve_1 = __importDefault(require("../../compiled/resolve"));
function addExt(opts) {
    const index = opts.file.lastIndexOf('.');
    return `${opts.file.slice(0, index)}${opts.ext}${opts.file.slice(index)}`;
}
exports.addExt = addExt;
function getAbsFiles(opts) {
    return opts.files.map((file) => {
        return (0, path_1.isAbsolute)(file) ? file : (0, path_1.join)(opts.cwd, file);
    });
}
exports.getAbsFiles = getAbsFiles;
/**
 * parse extends option for config
 */
function parseExtendsConfig(opts) {
    let { config } = opts;
    const { service, resolvePaths = service.configManager.files.map(path_1.dirname) } = opts;
    if (config.extends) {
        let absExtendsPath = '';
        const ConfigManager = service.configManager.constructor;
        // try to resolve extends path
        resolvePaths.some((dir) => {
            try {
                absExtendsPath = resolve_1.default.sync(config.extends, {
                    basedir: dir,
                    extensions: ['.js', '.ts'],
                });
                return true;
            }
            catch { }
        });
        if (!absExtendsPath) {
            throw new Error(`Cannot find extends config file: ${config.extends}`);
        }
        else if (service.configManager.files.includes(absExtendsPath)) {
            throw new Error(`Cannot extends config circularly for file: ${absExtendsPath}`);
        }
        // load extends config
        const { config: extendsConfig, files: extendsFiles } = ConfigManager.getUserConfig({ configFiles: [absExtendsPath] });
        // try to parse nested extends config
        const nestedConfig = parseExtendsConfig({
            config: extendsConfig,
            resolvePaths: [(0, path_1.dirname)(absExtendsPath)],
            service,
        });
        // merge extends config & save related files
        config = (0, deepmerge_1.default)(nestedConfig, config);
        service.configManager.files.push(...extendsFiles);
    }
    return config;
}
exports.parseExtendsConfig = parseExtendsConfig;
