"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = exports.getPkg = void 0;
const path_1 = require("path");
const fs_extra_1 = __importDefault(require("../../compiled/fs-extra"));
const loaders_1 = require("../builder/bundless/loaders");
const javascript_1 = require("../builder/bundless/loaders/javascript");
const config_1 = require("../config/config");
const utils_1 = require("../config/utils");
const constants_1 = require("../constants");
const types_1 = require("../types");
const _rDefault = (r) => r.default || r;
function getPkg({ cwd }) {
    // get pkg from package.json
    let pkg = {};
    let pkgPath = '';
    try {
        const filePath = (0, path_1.join)(cwd, 'package.json');
        pkg = fs_extra_1.default.readJSONSync(filePath);
        pkgPath = filePath;
    }
    catch (_e) {
        // APP_ROOT
        if (cwd !== process.cwd()) {
            try {
                const filePath = (0, path_1.join)(process.cwd(), 'package.json');
                pkg = fs_extra_1.default.readJSONSync(filePath);
                pkgPath = filePath;
            }
            catch (_e) {
                console.error(_e);
            }
        }
    }
    return { pkg, pkgPath: pkgPath || (0, path_1.join)(cwd, 'package.json') };
}
exports.getPkg = getPkg;
class Service {
    constructor(opts) {
        this.commands = {};
        this.config = {};
        this.userConfig = {};
        this.pkg = {};
        this.pkgPath = '';
        this.configManager = null;
        this.opts = opts;
        this.cwd = opts.cwd;
        this.commands = {
            dev: require('../commands/dev').default,
            build: require('../commands/build').default,
            version: require('../commands/version').default,
        };
    }
    async runCommand(opts) {
        let name = opts.name;
        if ((opts === null || opts === void 0 ? void 0 : opts.args.version) || name === 'v') {
            name = 'version';
        }
        else if ((opts === null || opts === void 0 ? void 0 : opts.args.help) || !name || name === 'h') {
            name = 'help';
        }
        const { args = {} } = opts;
        args._ = args._ || [];
        if (args._[0] === name)
            args._.shift();
        this.name = name;
        const { pkg, pkgPath } = getPkg({ cwd: this.cwd });
        this.pkg = pkg;
        this.pkgPath = pkgPath;
        const configManager = new config_1.Config({
            cwd: this.cwd,
            // env: this.env,
            defaultConfigFiles: constants_1.DEFAULT_CONFIG_FILES,
            specifiedEnv: process.env[`${constants_1.FRAMEWORK_NAME}_ENV`.toUpperCase()],
        });
        this.configManager = configManager;
        this.userConfig = configManager.getUserConfig().config;
        this.config = (0, utils_1.parseExtendsConfig)({
            config: configManager.getConfig().config,
            service: this,
        });
        await this.initLoaders();
        const command = this.commands[name];
        const res = await command.fn({ args, api: this });
        return res;
    }
    async initLoaders() {
        const bundlessLoaders = [
            {
                id: 'js-loader',
                test: /((?<!\.d)\.ts|\.(jsx?|tsx))$/,
                loader: require.resolve('../builder/bundless/loaders/javascript'),
            },
        ];
        bundlessLoaders.forEach((l) => (0, loaders_1.addLoader)(l));
        // collect all bundless js transformers
        const jsTransformers = [
            {
                id: types_1.JSTransformerTypes.BABEL,
                transformer: require.resolve('../builder/bundless/loaders/javascript/babel'),
            },
            {
                id: types_1.JSTransformerTypes.ESBUILD,
                transformer: require.resolve('../builder/bundless/loaders/javascript/esbuild'),
            },
        ];
        // register js transformers
        jsTransformers.forEach((t) => (0, javascript_1.addTransformer)(t));
    }
}
exports.Service = Service;
